
#include "call_em.h"
#include <sys/dirent.h>

#define L_TYPE			"/LCK.L."

/* Taylor and BNU use a 10 byte ascii number terminated with a '\n' */
#define SIZEOFPID 10

extern int debug;

char pri_device[NAME_MAX];
char alt_device[NAME_MAX];
char pri_lock[NAME_MAX];
char alt_lock[NAME_MAX];

int set_devices(char *);
int set_locks();
int zap_locks();
int check_locks(void);
int check_id(char *);
int reset_locks(int);

/*
 * Check if the command line entry references a 'tty' or a 'cua'. Puts
 * the appropiate names into 'device[]' and 'alt_device[]'. Return 0 if
 * it is else it returns -1. A -1 return infers that the command line
 * argument is the path name for a 'gettydefs' type file for use by the
 * '-c' option.
 */
int
set_devices(char *name)
{
	char *p, *q, *r;
	char *pri, *alt;
	char *cua = "cua";
	char *tty = "ttyS";

	/* Check if there is an existing name in pri_lock[] */
	p = pri_lock;
	q = tty;
	while( *p && *q ) {
		if( *p++ == *q ) q++;
		else q = tty;
	}
	p = pri_lock;
	r = cua;
	while( *p && *r ) {
		if( *p++ == *r ) r++;
		else r = cua;
	}
	if( ! *q || ! *r )
			zap_locks();

	q = name;
	r = tty;
	while (*q != *r && *q)
		q++;
	while (*q == *r && *q)
		q++, r++;
	if (*r) {
		q = name;
		r = cua;
		while (*q != *r && *q)
			q++;
		while (*q == *r && *q)
			q++, r++;
		if (*r)
			return (-1);
		pri = cua;
		alt = tty;
	} else {
		pri = tty;
		alt = cua;
	}
	strcat(strcat(strcat(strcpy(pri_lock, LCKDIR), L_TYPE), pri), q);
	strcat(strcat(strcpy(pri_device, "/dev/"), pri), q);
	strcat(strcat(strcat(strcpy(alt_lock, LCKDIR), L_TYPE), alt), q);
	strcat(strcat(strcpy(alt_device, "/dev/"), alt), q);
	if (debug > 3)
		fprintf(stderr, "Devices are '%s' and '%s'.\n",
				  pri_device, alt_device);
	return (0);
}

/*
 * Checks for existing LCK.. files and calls check_id() to examine the
 * contents.
 */
int
check_locks()
{
	int i, j, k;

	if ((k = access(pri_lock, F_OK)) < 0)
		i = 0;
	else
		i = check_id(pri_lock);
	if ((k = access(alt_lock, F_OK)) < 0)
		j = 0;
	else
		j = check_id(alt_lock);
	if (debug > 4) {
		fprintf(stderr, "check_locks: %s %sset, %d\n",
				  pri_lock, i ? "" : "un", i);
		fprintf(stderr, "check_locks: %s %sset, %d\n",
				  alt_lock, j ? "" : "un", j);
	}
	return (i + j);
}

/*
 * Used to check the id found in existing LCK.. files. Return: -1 -
 * Fail 0	- This is our LCK file
 */
int
check_id(char *path)
{
	int fd;
	char buf[SIZEOFPID + 2];
	int ret, j;
	pid_t i;

	if ((fd = open(path, O_RDONLY)) < 0) {
		if (debug)
			fprintf(stderr, "Unable to open \"%s\", %s\n",
					  path, sys_errlist[errno]);
		return (-1);
	}
	read(fd, &buf, sizeof(buf));
	i = atoi(buf);
	ret = 0;
	if (i != getpid()) {
		if ((j = kill(i, 0)) < 0)
			unlink(path);
		else
			ret = -1;
		if (debug > 3)
			fprintf(stderr,
					  "check_id: %s my pid %d, file pid %d, %s, ret %d\n",
					  path, getpid(), i, j < 0 ? "dead" : "active", ret);
	}
	close(fd);
	return (ret);
}

int
set_locks()
{
	FILE *lock;
	char tmp_lock[NAME_MAX];
	int pid, ret;

	pid = getpid();
	sprintf(tmp_lock, "%s%s%d", LCKDIR, L_TYPE, pid);
	tmp_lock[NAME_MAX - 2] = '\n';
	tmp_lock[NAME_MAX - 1] = '\0';
	if (debug > 6)
		fprintf(stderr, "set_locks: tmp_lock %s\n", tmp_lock);
	if ((lock = fopen(tmp_lock, "w")) == NULL) {
		fprintf(stderr, "Unable to open %s, %s\n",
				  tmp_lock, sys_errlist[errno]);
		return (-1);
	}
	fprintf(lock, "%*d\n", SIZEOFPID, pid);
	fclose(lock);

	/*
	 * We checked for locks before we started but it could have been a
	 * long while before this call came in.  However, the device must
	 * have been clear or else this call couldn't have come. :-)
	 */
	ret = -1;
	if (check_locks() == 0) {
		link(tmp_lock, pri_lock);
		link(tmp_lock, alt_lock);
		ret = 0;
	}
	unlink(tmp_lock);
	if (debug > 2)
		fprintf(stderr, "\tset_locks: %s set.\n", pri_lock);
	return (ret);
}

/*
 * This is for changing the pid stored in a lock file after a fork().
 */
int
reset_locks(int new_id)
{
	int fd;
	int i;
	char buf[SIZEOFPID + 2];

	if(( fd = open( pri_lock, O_RDWR )) < 0 ) {
		fprintf( stderr, "Unable to open '%s', %s\n",
				  pri_lock, sys_errlist[errno] );
		return( -1 );
	}
	read( fd, &buf, sizeof(buf) );
	i = atoi( buf );
	if( i != getpid()) {
		fprintf(stderr, "Someone stole our lock and left %d instead of %d\n",
				  i, getpid());
		return( -1 );
	}
	sprintf(buf, "%*d\n", SIZEOFPID, new_id);
	lseek( fd, 0, 0 );
	i = strlen( buf );
	write( fd, &buf, i );
	close( fd );
	return (0);
}

int
zap_locks()
{
	if (check_locks() == 0) {
		unlink(pri_lock);
		unlink(alt_lock);
		return (0);
	}
	return (-1);
}
